@@ -36,7 +36,7 @@ import java.util.Set;
  * <a href="DependencyVisitor.html">DependencyVisitor</a>.
  *
  * @author <a href="mailto:markhobson@gmail.com">Mark Hobson</a>
- * @version $Id: DependencyClassFileVisitor.java 1589585 2014-04-24 04:58:35Z olamy $
+ * @version $Id: DependencyClassFileVisitor.java 1717974 2015-12-04 14:48:05Z krosenvold $
  * @see #getDependencies()
  */
 public class DependencyClassFileVisitor
@@ -64,6 +64,12 @@ public class DependencyClassFileVisitor
         {
             ClassReader reader = new ClassReader( in );
 
+            final Set<String> constantPoolClassRefs = ConstantPoolParser.getConstantPoolClassReferences( reader.b );
+            for ( String string : constantPoolClassRefs )
+            {
+                resultCollector.addName( string );
+            }
+
             AnnotationVisitor annotationVisitor = new DefaultAnnotationVisitor( resultCollector );
             SignatureVisitor signatureVisitor = new DefaultSignatureVisitor( resultCollector );
             FieldVisitor fieldVisitor = new DefaultFieldVisitor( annotationVisitor, resultCollector );
