@@ -19,14 +19,7 @@ package org.apache.maven.shared.dependency.analyzer;
  * under the License.
  */
 
-import java.io.File;
-import java.util.Arrays;
-import java.util.Collections;
-import java.util.HashSet;
-import java.util.List;
-import java.util.Properties;
-import java.util.Set;
-
+import org.apache.commons.lang.SystemUtils;
 import org.apache.maven.artifact.Artifact;
 import org.apache.maven.artifact.DefaultArtifact;
 import org.apache.maven.artifact.handler.ArtifactHandler;
@@ -40,12 +33,19 @@ import org.apache.maven.shared.test.plugin.ProjectTool;
 import org.apache.maven.shared.test.plugin.RepositoryTool;
 import org.apache.maven.shared.test.plugin.TestToolsException;
 import org.codehaus.plexus.PlexusTestCase;
+import java.io.File;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.HashSet;
+import java.util.List;
+import java.util.Properties;
+import java.util.Set;
 
 /**
  * Tests <code>DefaultProjectDependencyAnalyzer</code>.
- * 
+ *
  * @author <a href="mailto:markhobson@gmail.com">Mark Hobson</a>
- * @version $Id: DefaultProjectDependencyAnalyzerTest.java 1635191 2014-10-29 16:33:52Z andham $
+ * @version $Id: DefaultProjectDependencyAnalyzerTest.java 1722332 2015-12-30 14:17:31Z andham $
  * @see DefaultProjectDependencyAnalyzer
  */
 public class DefaultProjectDependencyAnalyzerTest
@@ -78,7 +78,8 @@ public class DefaultProjectDependencyAnalyzerTest
         if ( localRepo == null )
         {
             RepositoryTool repositoryTool = (RepositoryTool) lookup( RepositoryTool.ROLE );
-            localRepo = repositoryTool.findLocalRepositoryDirectory().getAbsoluteFile();
+            localRepo = repositoryTool.findLocalRepositoryDirectory()
+                                      .getAbsoluteFile();
             System.out.println( "Local repository: " + localRepo );
         }
 
@@ -115,6 +116,56 @@ public class DefaultProjectDependencyAnalyzerTest
         assertEquals( expectedAnalysis, actualAnalysis );
     }
 
+    public void testJava8methodRefs()
+        throws TestToolsException, ProjectDependencyAnalyzerException
+    {
+        if ( !SystemUtils.isJavaVersionAtLeast( 180 ) )
+        {
+            return;
+        }
+
+        // Only visible through constant pool analysis (supported for JDK8+)
+        compileProject( "java8methodRefs/pom.xml" );
+
+        MavenProject project = getProject( "java8methodRefs/pom.xml" );
+
+        ProjectDependencyAnalysis actualAnalysis = analyzer.analyze( project );
+
+        Artifact project1 = createArtifact( "commons-io", "commons-io", "jar", "2.4", "compile" );
+        Artifact project2 = createArtifact( "commons-lang", "commons-lang", "jar", "2.6", "compile" );
+        Set<Artifact> usedDeclaredArtifacts = new HashSet<Artifact>( Arrays.asList( project1, project2 ) );
+
+        ProjectDependencyAnalysis expectedAnalysis =
+            new ProjectDependencyAnalysis( usedDeclaredArtifacts, new HashSet<Artifact>(), new HashSet<Artifact>() );
+
+        assertEquals( expectedAnalysis, actualAnalysis );
+    }
+
+
+    public void testInlinedStaticReference()
+        throws TestToolsException, ProjectDependencyAnalyzerException
+    {
+        if ( !SystemUtils.isJavaVersionAtLeast( 180 ) )
+        {
+            return;
+        }
+
+        // Only visible through constant pool analysis (supported for JDK8+)
+        compileProject( "inlinedStaticReference/pom.xml" );
+
+        MavenProject project = getProject( "inlinedStaticReference/pom.xml" );
+
+        ProjectDependencyAnalysis actualAnalysis = analyzer.analyze( project );
+
+        Artifact project1 = createArtifact( "dom4j", "dom4j", "jar", "1.6.1", "compile" );
+        Set<Artifact> usedDeclaredArtifacts = Collections.singleton( project1 );
+
+        ProjectDependencyAnalysis expectedAnalysis =
+            new ProjectDependencyAnalysis( usedDeclaredArtifacts, new HashSet<Artifact>(), new HashSet<Artifact>() );
+
+        assertEquals( expectedAnalysis, actualAnalysis );
+    }
+
     public void testJarWithCompileDependency()
         throws TestToolsException, ProjectDependencyAnalyzerException
     {
@@ -125,8 +176,8 @@ public class DefaultProjectDependencyAnalyzerTest
         if ( project2.getBuild().getOutputDirectory().contains( "${" ) )
         {
             // if Maven version used as dependency is upgraded to >= 2.2.0 
-            throw new TestToolsException( "output directory was not interpolated: "
-                + project2.getBuild().getOutputDirectory() );
+            throw new TestToolsException( "output directory was not interpolated: " + project2.getBuild()
+                                                                                              .getOutputDirectory() );
         }
 
         ProjectDependencyAnalysis actualAnalysis = analyzer.analyze( project2 );
@@ -140,6 +191,40 @@ public class DefaultProjectDependencyAnalyzerTest
         assertEquals( expectedAnalysis, actualAnalysis );
     }
 
+    public void testForceDeclaredDependenciesUsage()
+        throws TestToolsException, ProjectDependencyAnalyzerException
+    {
+        compileProject( "jarWithTestDependency/pom.xml" );
+
+        MavenProject project2 = getProject( "jarWithTestDependency/project2/pom.xml" );
+
+        ProjectDependencyAnalysis analysis = analyzer.analyze( project2 );
+
+        try
+        {
+            analysis.forceDeclaredDependenciesUsage( new String[] {
+                "org.apache.maven.shared.dependency-analyzer.tests:jarWithTestDependency1" } );
+            fail( "failure expected since junit dependency is declared-used" );
+        }
+        catch ( ProjectDependencyAnalyzerException pdae )
+        {
+            assertTrue( pdae.getMessage().contains( "Trying to force use of dependencies which are "
+                + "declared but already detected as used: "
+                + "[org.apache.maven.shared.dependency-analyzer.tests:jarWithTestDependency1]" ) );
+        }
+
+        try
+        {
+            analysis.forceDeclaredDependenciesUsage( new String[]{ "undefined:undefined" } );
+            fail( "failure expected since undefined dependency is not declared" );
+        }
+        catch ( ProjectDependencyAnalyzerException pdae )
+        {
+            assertTrue( pdae.getMessage().contains( "Trying to force use of dependencies which are "
+                + "not declared: [undefined:undefined]" ) );
+        }
+    }
+
     public void testJarWithTestDependency()
         throws TestToolsException, ProjectDependencyAnalyzerException
     {
@@ -150,48 +235,25 @@ public class DefaultProjectDependencyAnalyzerTest
         ProjectDependencyAnalysis actualAnalysis = analyzer.analyze( project2 );
 
         Artifact project1 =
-            createArtifact( "org.apache.maven.shared.dependency-analyzer.tests", "jarWithTestDependency1", "jar",
-                            "1.0", "test" );
-        Set<Artifact> usedDeclaredArtifacts = Collections.singleton( project1 );
-
+            createArtifact( "org.apache.maven.shared.dependency-analyzer.tests", "jarWithTestDependency1", "jar", "1.0",
+                            "test" );
         Artifact junit = createArtifact( "junit", "junit", "jar", "3.8.1", "test" );
-        Set<Artifact> unusedDeclaredArtifacts = Collections.singleton( junit );
 
-        ProjectDependencyAnalysis expectedAnalysis =
-            new ProjectDependencyAnalysis( usedDeclaredArtifacts, null, unusedDeclaredArtifacts );
+        ProjectDependencyAnalysis expectedAnalysis;
+        if ( SystemUtils.isJavaVersionAtLeast( 180 ) )
+        {
+            Set<Artifact> usedDeclaredArtifacts = new HashSet<Artifact>( Arrays.asList( project1, junit ) );
+            expectedAnalysis = new ProjectDependencyAnalysis( usedDeclaredArtifacts, null, null );
+        }
+        else
+        {
+            // With JDK 7 and earlier, not all deps are identified correctly
+            Set<Artifact> usedDeclaredArtifacts = Collections.singleton( project1 );
+            Set<Artifact> unusedDeclaredArtifacts = Collections.singleton( junit );
+            expectedAnalysis = new ProjectDependencyAnalysis( usedDeclaredArtifacts, null, unusedDeclaredArtifacts );
+        }
 
         assertEquals( expectedAnalysis, actualAnalysis );
-
-        // MSHARED-253: force used dependency (which is actually used but not detected)
-        ProjectDependencyAnalysis forcedAnalysis =
-            actualAnalysis.forceDeclaredDependenciesUsage( new String[] { "junit:junit" } );
-
-        usedDeclaredArtifacts = new HashSet<Artifact>( Arrays.asList( project1, junit ) );
-        expectedAnalysis = new ProjectDependencyAnalysis( usedDeclaredArtifacts, null, null );
-
-        assertEquals( expectedAnalysis, forcedAnalysis );
-
-        try
-        {
-            forcedAnalysis.forceDeclaredDependenciesUsage( new String[]{ "junit:junit" } );
-            fail( "failure expected since junit dependency is declared-used" );
-        }
-        catch( ProjectDependencyAnalyzerException pdae )
-        {
-            assertTrue( pdae.getMessage().contains( "Trying to force use of dependencies which are "
-                                                        + "declared but already detected as used: [junit:junit]" ) );
-        }
-
-        try
-        {
-            forcedAnalysis.forceDeclaredDependenciesUsage( new String[]{ "undefined:undefined" } );
-            fail( "failure expected since undefined dependency is not declared" );
-        }
-        catch( ProjectDependencyAnalyzerException pdae )
-        {
-            assertTrue( pdae.getMessage().contains( "Trying to force use of dependencies which are "
-                                                        + "not declared: [undefined:undefined]" ) );
-        }
     }
 
     public void testJarWithXmlTransitiveDependency()
@@ -206,8 +268,7 @@ public class DefaultProjectDependencyAnalyzerTest
         Artifact jdom = createArtifact( "dom4j", "dom4j", "jar", "1.6.1", "compile" );
         Set<Artifact> usedDeclaredArtifacts = Collections.singleton( jdom );
 
-        ProjectDependencyAnalysis expectedAnalysis =
-            new ProjectDependencyAnalysis( usedDeclaredArtifacts, null, null );
+        ProjectDependencyAnalysis expectedAnalysis = new ProjectDependencyAnalysis( usedDeclaredArtifacts, null, null );
 
         // MSHARED-47: usedUndeclaredArtifacts=[xml-apis:xml-apis:jar:1.0.b2:compile]
         //assertEquals( expectedAnalysis, actualAnalysis );
@@ -221,11 +282,11 @@ public class DefaultProjectDependencyAnalyzerTest
         // difficult to create multi-module project with Maven 2.x, so here's hacky solution
         // to get a inter-module dependency
         MavenProject project = getProject( "multimoduleProject/module2/pom.xml" );
-        @SuppressWarnings( "unchecked" )
-        Set<Artifact> dependencyArtifacts = project.getArtifacts();
+        @SuppressWarnings( "unchecked" ) Set<Artifact> dependencyArtifacts = project.getArtifacts();
         for ( Artifact artifact : dependencyArtifacts )
         {
-            if ( artifact.getArtifactId().equals( "test-module1" ) )
+            if ( artifact.getArtifactId()
+                         .equals( "test-module1" ) )
             {
                 File dir = getTestFile( "target/test-classes/", "multimoduleProject/module1/target/classes/" );
                 artifact.setFile( dir );
@@ -237,8 +298,7 @@ public class DefaultProjectDependencyAnalyzerTest
         Artifact junit = createArtifact( "org.apache.maven.its.dependency", "test-module1", "jar", "1.0", "compile" );
         Set<Artifact> usedDeclaredArtifacts = Collections.singleton( junit );
 
-        ProjectDependencyAnalysis expectedAnalysis =
-            new ProjectDependencyAnalysis( usedDeclaredArtifacts, null, null );
+        ProjectDependencyAnalysis expectedAnalysis = new ProjectDependencyAnalysis( usedDeclaredArtifacts, null, null );
 
         assertEquals( expectedAnalysis, actualAnalysis );
     }
